@@ -15,15 +15,13 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GROONGA_IN_H
-#include "groonga_in.h"
-#endif /* GROONGA_IN_H */
+#include "grn.h"
 
 #include <string.h>
-#include "str.h"
-#include "db.h"
-#include "util.h"
-#include "output.h"
+#include "grn_str.h"
+#include "grn_db.h"
+#include "grn_util.h"
+#include "grn_output.h"
 
 #define LEVELS (&ctx->impl->levels)
 #define DEPTH (GRN_BULK_VSIZE(LEVELS)>>2)
@@ -1023,6 +1021,7 @@ grn_output_table_header(grn_ctx *ctx, grn_obj *outbuf,
                         grn_content_type output_type,
                         grn_obj *table, grn_obj_format *format)
 {
+  if (format->nhits != -1) {
     grn_output_array_open(ctx, outbuf, output_type, "NHITS", 1);
     if (output_type == GRN_CONTENT_XML) {
       grn_text_itoa(ctx, outbuf, format->nhits);
@@ -1030,6 +1029,7 @@ grn_output_table_header(grn_ctx *ctx, grn_obj *outbuf,
       grn_output_int32(ctx, outbuf, output_type, format->nhits);
     }
     grn_output_array_close(ctx, outbuf, output_type);
+  }
 }
 
 static inline int
@@ -1116,7 +1116,11 @@ grn_output_table_column(grn_ctx *ctx, grn_obj *outbuf,
     GRN_BULK_REWIND(buf);
     grn_column_name_(ctx, column, buf);
     grn_output_obj(ctx, outbuf, output_type, buf, NULL);
-    range_id = grn_obj_get_range(ctx, column);
+    if (column->header.type == GRN_COLUMN_INDEX) {
+      range_id = GRN_DB_UINT32;
+    } else {
+      range_id = grn_obj_get_range(ctx, column);
+    }
     if (range_id == GRN_ID_NIL) {
       grn_output_cstr(ctx, outbuf, output_type, "null");
     } else {
@@ -1138,6 +1142,46 @@ grn_output_table_column(grn_ctx *ctx, grn_obj *outbuf,
   grn_output_array_close(ctx, outbuf, output_type);
 }
 
+static inline void
+grn_output_table_column_by_expression(grn_ctx *ctx, grn_obj *outbuf,
+                                      grn_content_type output_type,
+                                      grn_expr_code *code,
+                                      grn_expr_code *code_end,
+                                      grn_obj *buf)
+{
+  if (code_end <= code) {
+    grn_output_array_open(ctx, outbuf, output_type, "COLUMN", 2);
+    grn_output_null(ctx, outbuf, output_type);
+    grn_output_null(ctx, outbuf, output_type);
+    grn_output_array_close(ctx, outbuf, output_type);
+    return;
+  }
+
+  switch (code_end[-1].op) {
+  case GRN_OP_GET_MEMBER :
+    if ((code_end - code) == 3) {
+      grn_output_array_open(ctx, outbuf, output_type, "COLUMN", 2);
+
+      GRN_BULK_REWIND(buf);
+      grn_column_name_(ctx, code[0].value, buf);
+      GRN_TEXT_PUTC(ctx, buf, '[');
+      grn_inspect(ctx, buf, code[1].value);
+      GRN_TEXT_PUTC(ctx, buf, ']');
+
+      grn_output_obj(ctx, outbuf, output_type, buf, NULL);
+      grn_output_null(ctx, outbuf, output_type);
+
+      grn_output_array_close(ctx, outbuf, output_type);
+    } else {
+      grn_output_table_column(ctx, outbuf, output_type, code->value, buf);
+    }
+    break;
+  default :
+    grn_output_table_column(ctx, outbuf, output_type, code->value, buf);
+    break;
+  }
+}
+
 static inline void
 grn_output_table_columns_by_expression(grn_ctx *ctx, grn_obj *outbuf,
                                        grn_content_type output_type,
@@ -1163,30 +1207,35 @@ grn_output_table_columns_by_expression(grn_ctx *ctx, grn_obj *outbuf,
     }
 
     have_comma = GRN_TRUE;
-    code_start_offset = previous_comma_offset + 1;
     if (is_first_comma) {
+      int n_used_codes;
       int code_end_offset;
-      int n_used_code;
 
-      grn_output_table_column(ctx, outbuf, output_type,
-                              expr->codes[0].value, buf);
+      n_used_codes = count_used_n_codes(ctx, expr->codes, code - 1);
+      code_end_offset = code - expr->codes - n_used_codes;
 
-      code_end_offset = code - expr->codes - code_start_offset - 1;
-      n_used_code = count_used_n_codes(ctx,
-                                       expr->codes,
-                                       expr->codes + code_end_offset);
-      code_start_offset = code_end_offset - n_used_code + 1;
+      grn_output_table_column_by_expression(ctx, outbuf, output_type,
+                                            expr->codes,
+                                            expr->codes + code_end_offset,
+                                            buf);
+      code_start_offset = code_end_offset;
       is_first_comma = GRN_FALSE;
+    } else {
+      code_start_offset = previous_comma_offset + 1;
     }
 
-    grn_output_table_column(ctx, outbuf, output_type,
-                            expr->codes[code_start_offset].value, buf);
+    grn_output_table_column_by_expression(ctx, outbuf, output_type,
+                                          expr->codes + code_start_offset,
+                                          code,
+                                          buf);
     previous_comma_offset = code - expr->codes;
   }
 
   if (!have_comma && expr->codes_curr > 0) {
-    grn_output_table_column(ctx, outbuf, output_type,
-                            expr->codes[0].value, buf);
+    grn_output_table_column_by_expression(ctx, outbuf, output_type,
+                                          expr->codes,
+                                          code_end,
+                                          buf);
   }
 
   grn_output_array_close(ctx, outbuf, output_type);
@@ -1411,6 +1460,7 @@ grn_output_obj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
     break;
   case GRN_TABLE_HASH_KEY :
   case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_DAT_KEY :
   case GRN_TABLE_NO_KEY :
     grn_output_table(ctx, outbuf, output_type, obj, format);
     break;
@@ -1454,7 +1504,10 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
   unsigned int len;
   int offset = 0, limit = 0, record_n = 0;
   int column_n = 0, column_text_n = 0, result_set_n = -1;
-  int in_vector = 0, first_vector_element = 0;
+  grn_bool in_vector = GRN_FALSE;
+  unsigned int vector_element_n = 0;
+  grn_bool in_weight_vector = GRN_FALSE;
+  unsigned int weight_vector_item_n = 0;
 
   s = GRN_TEXT_VALUE(output);
   e = GRN_BULK_CURR(output);
@@ -1508,8 +1561,15 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
           }
         } else if (EQUAL_NAME_P("VECTOR")) {
           char *c = transform_xml_next_column(&columns, column_n++);
-          in_vector = 1;
-          first_vector_element = 1;
+          in_vector = GRN_TRUE;
+          vector_element_n = 0;
+          GRN_TEXT_PUTS(ctx, transformed, "<FIELD NAME=\"");
+          GRN_TEXT_PUTS(ctx, transformed, c);
+          GRN_TEXT_PUTS(ctx, transformed, "\">");
+        } else if (EQUAL_NAME_P("WEIGHT_VECTOR")) {
+          char *c = transform_xml_next_column(&columns, column_n++);
+          in_weight_vector = GRN_TRUE;
+          weight_vector_item_n = 0;
           GRN_TEXT_PUTS(ctx, transformed, "<FIELD NAME=\"");
           GRN_TEXT_PUTS(ctx, transformed, c);
           GRN_TEXT_PUTS(ctx, transformed, "\">");
@@ -1538,25 +1598,44 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
                         "</SEGMENT>\n"
                         "</SEGMENTS>\n");
         } else if (EQUAL_NAME_P("VECTOR")) {
-          in_vector = 0;
-          first_vector_element = 0;
+          in_vector = GRN_FALSE;
+          GRN_TEXT_PUTS(ctx, transformed, "</FIELD>\n");
+        } else if (EQUAL_NAME_P("WEIGHT_VECTOR")) {
+          in_weight_vector = GRN_FALSE;
           GRN_TEXT_PUTS(ctx, transformed, "</FIELD>\n");
         } else {
           switch (place) {
           case XML_PLACE_HIT :
             if (result_set_n == 0) {
-              if (!in_vector) {
+              if (in_vector) {
+                if (vector_element_n > 0) {
+                  GRN_TEXT_PUTS(ctx, transformed, ", ");
+                }
+                GRN_TEXT_PUT(ctx, transformed,
+                             GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
+                vector_element_n++;
+              } else if (in_weight_vector) {
+                grn_bool is_key;
+                is_key = ((weight_vector_item_n % 2) == 0);
+                if (is_key) {
+                  unsigned int weight_vector_key_n;
+                  weight_vector_key_n = weight_vector_item_n / 2;
+                  if (weight_vector_key_n > 0) {
+                    GRN_TEXT_PUTS(ctx, transformed, ", ");
+                  }
+                } else {
+                  GRN_TEXT_PUTS(ctx, transformed, ":");
+                }
+                GRN_TEXT_PUT(ctx, transformed,
+                             GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
+                weight_vector_item_n++;
+              } else {
                 char *c = transform_xml_next_column(&columns, column_n++);
                 GRN_TEXT_PUTS(ctx, transformed, "<FIELD NAME=\"");
                 GRN_TEXT_PUTS(ctx, transformed, c);
                 GRN_TEXT_PUTS(ctx, transformed, "\">");
-              }
-              if (in_vector && !first_vector_element) {
-                GRN_TEXT_PUTS(ctx, transformed, ", ");
-              }
-              GRN_TEXT_PUT(ctx, transformed,
-                           GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
-              if (!in_vector) {
+                GRN_TEXT_PUT(ctx, transformed,
+                             GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
                 GRN_TEXT_PUTS(ctx, transformed, "</FIELD>\n");
               }
             } else {
@@ -1567,7 +1646,6 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
                            GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
               GRN_TEXT_PUTS(ctx, transformed, "\" ");
             }
-            first_vector_element = 0;
             break;
           default :
             if (EQUAL_NAME_P("NHITS")) {
