@@ -15,17 +15,17 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include "groonga_in.h"
-#include "groonga/plugin.h"
+#include "grn.h"
+#include <groonga/plugin.h>
 
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "db.h"
-#include "plugin_in.h"
-#include "ctx_impl.h"
-#include "util.h"
+#include "grn_db.h"
+#include "grn_plugin.h"
+#include "grn_ctx_impl.h"
+#include "grn_util.h"
 
 static grn_hash *grn_plugins = NULL;
 static grn_critical_section grn_plugins_lock;
@@ -89,6 +89,10 @@ grn_plugin_path(grn_ctx *ctx, grn_id id)
   const char *system_plugins_dir;
   size_t system_plugins_dir_size;
 
+  if (id == GRN_ID_NIL) {
+    return NULL;
+  }
+
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
   path = _grn_hash_key(&grn_gctx, grn_plugins, id, &key_size);
   CRITICAL_SECTION_LEAVE(grn_plugins_lock);
@@ -646,6 +650,12 @@ grn_plugin_proc_alloc(grn_ctx *ctx, grn_user_data *user_data,
   return grn_proc_alloc(ctx, user_data, domain, flags);
 }
 
+grn_obj *
+grn_plugin_proc_get_vars(grn_ctx *ctx, grn_user_data *user_data)
+{
+  return grn_proc_get_vars(ctx, user_data);
+}
+
 grn_obj *
 grn_plugin_proc_get_var(grn_ctx *ctx, grn_user_data *user_data,
                         const char *name, int name_size)
