@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
-/* Copyright(C) 2009 Brazil
+/* Copyright(C) 2009-2014 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "db.h"
+#include "grn_db.h"
 #include <stdio.h>
 
 #include <gcutter.h>
@@ -42,7 +42,7 @@ void test_value_access(void);
 void test_snip(void);
 void test_snip_without_tags(void);
 void test_float_literal(void);
-void test_uint32_literal(void);
+void test_int32_literal(void);
 void test_lager_than_int32_literal(void);
 void test_int64_literal(void);
 void test_long_integer_literal(void);
@@ -287,11 +287,30 @@ test_parse(gconstpointer data)
   PARSE(cond, gcut_data_get_string(data, "query_poyo"),
               gcut_data_get_int(data, "query_poyo_parse_level"));
   grn_expr_append_op(&context, cond, GRN_OP_AND, 2);
-  grn_test_assert_expr(&context,
-                       "noname($1:0){2body GET_VALUE,0\"hoge\",4MATCH,"
-                                    "2body GET_VALUE,0\"moge\",0MATCH,4AND,"
-                                    "2body GET_VALUE,0\"poyo\",0MATCH,0AND}",
-                       cond);
+  grn_test_assert_expr(
+    &context,
+    "#<expr\n"
+    "  vars:{\n"
+    "    $1:#<record:no_key:docs id:0(nonexistent)>\n"
+    "  },\n"
+    "  codes:{\n"
+    "    0:<get_value(), modify:2, "
+    "value:#<column:var_size docs.body range:Text type:scalar compress:none>>,\n"
+    "    1:<push(), modify:0, value:\"hoge\">,\n"
+    "    2:<match(), modify:4, value:(NULL)>,\n"
+    "    3:<get_value(), modify:2, "
+    "value:#<column:var_size docs.body range:Text type:scalar compress:none>>,\n"
+    "    4:<push(), modify:0, value:\"moge\">,\n"
+    "    5:<match(), modify:0, value:(NULL)>,\n"
+    "    6:<and(), modify:4, value:(NULL)>,\n"
+    "    7:<get_value(), modify:2, "
+    "value:#<column:var_size docs.body range:Text type:scalar compress:none>>,\n"
+    "    8:<push(), modify:0, value:\"poyo\">,\n"
+    "    9:<match(), modify:0, value:(NULL)>,\n"
+    "    10:<and(), modify:0, value:(NULL)>\n"
+    "  }\n"
+    ">",
+    cond);
   res = grn_table_create(&context, NULL, 0, NULL,
                          GRN_TABLE_HASH_KEY|GRN_OBJ_WITH_SUBREC, docs, NULL);
   cut_assert_not_null(res);
@@ -315,9 +334,20 @@ test_parse(gconstpointer data)
   GRN_RECORD_INIT(v, 0, grn_obj_id(&context, docs));
   PARSE(cond, gcut_data_get_string(data, "query_size"),
               gcut_data_get_int(data, "query_size_parse_level"));
-  grn_test_assert_expr(&context,
-                       "noname($1:0){2size GET_VALUE,014,0EQUAL}",
-                       cond);
+  grn_test_assert_expr(
+    &context,
+    "#<expr\n"
+    "  vars:{\n"
+    "    $1:#<record:no_key:docs id:0(nonexistent)>\n"
+    "  },\n"
+    "  codes:{\n"
+    "    0:<get_value(), modify:2, "
+    "value:#<column:fix_size docs.size range:UInt32 type:scalar compress:none>>,\n"
+    "    1:<push(), modify:0, value:14>,\n"
+    "    2:<equal(), modify:0, value:(NULL)>\n"
+    "  }\n"
+    ">",
+    cond);
   res = grn_table_create(&context, NULL, 0, NULL,
                          GRN_TABLE_HASH_KEY|GRN_OBJ_WITH_SUBREC, docs, NULL);
   cut_assert_not_null(res);
@@ -662,7 +692,7 @@ test_snip(void)
         GRN_EXPR_SYNTAX_QUERY|GRN_EXPR_ALLOW_PRAGMA);
 
   {
-    grn_snip *snip;
+    grn_obj *snip;
     int flags = 0;
     unsigned int width, max_results;
     const char *open_tags[] = {"[[", "<"};
@@ -709,7 +739,7 @@ test_snip(void)
                             result);
     cut_assert_equal_uint(49, result_len);
 
-    grn_test_assert(grn_snip_close(&context, snip));
+    grn_test_assert(grn_obj_close(&context, snip));
   }
 }
 
@@ -727,7 +757,7 @@ test_snip_without_tags(void)
         GRN_EXPR_SYNTAX_QUERY|GRN_EXPR_ALLOW_PRAGMA);
 
   {
-    grn_snip *snip;
+    grn_obj *snip;
     int flags = 0;
     unsigned int width, max_results;
     unsigned int n_results;
@@ -770,7 +800,7 @@ test_snip_without_tags(void)
                             result);
     cut_assert_equal_uint(45, result_len);
 
-    grn_test_assert(grn_snip_close(&context, snip));
+    grn_test_assert(grn_obj_close(&context, snip));
   }
 }
 
@@ -810,14 +840,14 @@ test_float_literal(void)
 }
 
 void
-test_uint32_literal(void)
+test_int32_literal(void)
 {
   grn_obj *var;
   const char *str_expr = "var = 123456";
 
   var = parse_numeric_literal(str_expr);
-  cut_assert_equal_int(GRN_DB_UINT32, GRN_OBJ_GET_DOMAIN(var));
-  cut_assert_equal_int(123456, GRN_UINT32_VALUE(var));
+  cut_assert_equal_int(GRN_DB_INT32, GRN_OBJ_GET_DOMAIN(var));
+  cut_assert_equal_int(123456, GRN_INT32_VALUE(var));
 }
 
 void
@@ -827,8 +857,8 @@ test_lager_than_int32_literal(void)
   const char *str_expr = "var = 3456789012";
 
   var = parse_numeric_literal(str_expr);
-  cut_assert_equal_int(GRN_DB_UINT32, GRN_OBJ_GET_DOMAIN(var));
-  cut_assert_equal_int(3456789012U, GRN_UINT32_VALUE(var));
+  cut_assert_equal_int(GRN_DB_INT64, GRN_OBJ_GET_DOMAIN(var));
+  cut_assert_equal_int(G_GINT64_CONSTANT(3456789012), GRN_INT64_VALUE(var));
 }
 
 void
@@ -867,7 +897,19 @@ test_syntax_equal_string_reference_key(void)
   cut_assert_not_null(cond);
   cut_assert_not_null(v);
   PARSE(cond, "author == \"morita\"", GRN_EXPR_SYNTAX_SCRIPT);
-  grn_test_assert_expr(&context,
-                       "noname($1:0){2author GET_VALUE,0\"morita\",0EQUAL}",
-                       cond);
+  grn_test_assert_expr(
+    &context,
+    "#<expr\n"
+    "  vars:{\n"
+    "    $1:#<record:no_key:docs id:0(nonexistent)>\n"
+    "  },\n"
+    "  codes:{\n"
+    "    0:<get_value(), modify:2, "
+    "value:#<column:fix_size docs.author range:properties type:scalar compress:none>>,\n"
+    "    1:<push(), modify:0, "
+    "value:#<record:hash:properties id:1 key:\"morita\">>,\n"
+    "    2:<equal(), modify:0, value:(NULL)>\n"
+    "  }\n"
+    ">",
+    cond);
 }
